#  tests for pandas-profiling-3.6.3-py39h6ffa863_0 (this is a generated file);
print('===== testing package: pandas-profiling-3.6.3-py39h6ffa863_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import os

import numpy as np
import pandas as pd
import pandas_profiling
from pandas_profiling import ProfileReport

df = pd.DataFrame(np.random.rand(100, 5), columns=["a", "b", "c", "d", "e"])

profile = ProfileReport(df, title="Pandas Profiling Report", explorative=True)

profile.to_file("report.html")

assert os.path.exists("report.html") == True
#  --- run_test.py (end) ---

print('===== pandas-profiling-3.6.3-py39h6ffa863_0 OK =====');
